package testing

// TagsListResponse represents a raw tags response.
const TagsListResponse = `
{
    "tags": ["foo", "bar", "baz"]
}
`

// TagsReplaceAllRequest represents a raw tags Replace request.
const TagsReplaceAllRequest = `
{
    "tags": ["tag1", "tag2", "tag3"]
}
`

// TagsReplaceAllResponse represents a raw tags Replace response.
const TagsReplaceAllResponse = `
{
    "tags": ["tag1", "tag2", "tag3"]
}
`
