package testing

const attributestagsReplaceAllRequest = `
{
	"tags": ["abc", "xyz"]
}
`

const attributestagsReplaceAllResult = `
{
	"tags": ["abc", "xyz"]
}
`

const attributestagsListResult = `
{
	"tags": ["abc", "xyz"]
}
`
