/*
Package apiversions provides information and interaction with the different
API versions for the OpenStack Block Storage service, code-named Cinder.

This package is deprecated and should only be used in environments where the
older API version format is expected.

Consider using the gophercloud/openstack/blockstorage/apiversions package
instead.
*/
package apiversions
